/*
        Maintened by *Edwards
        For FantasiaShard.com

        Edwards@FantasiaShard.com

        2009-08-17
        
        Last Modifications 2009-09-29
             * Added AddArticle() when renaming item

        Last Modifications 2009-10-03
             * Fixed problem when Id warding items that has charges instead of levels
*/
use uo;
use os;
use cfgfile;

include ":attributes:attributes";
include ":equipment:equipment";
include ":loot:identify_ex";

function NameByEnchant( item, cursed:=0 )

	EraseObjProperty( item, "UnIDedMagicalItem" );
      
        var base_name := GetObjProperty( item, "BaseName" );
        if( !base_name )
                var item_cfg := ReadConfigFile( ":*:itemdesc" ),
	            item_name := item_cfg[item.objtype].desc;

		SetName( item, item_name );
	else
		SetName( item, base_name );
	endif

        var enchanted_name := GetObjProperty( item, "Enchanted" );;
	if( enchanted_name )
		SetName( item, enchanted_name+" "+item.desc );
	endif

        var enchanted_color := GetObjProperty( item, "EnchantColor" );
	if( enchanted_color )
		item.color := enchanted_color;
                EraseObjProperty( item, "EnchantColor" );
	endif

        var isGMItem := GetObjProperty( item, "IsGMItem" );
	if( isGMItem )
		SetName( item, isGMItem );
		return 1;
	endif

	var hitScript := GetObjProperty( item, "HitScriptNumber" );
	if( hitScript )
		RenameByHitscript( item, hitScript, cursed );
	endif

	var onHitScript := GetObjProperty( item, "OnHitScriptNumber" );
	if( onHitScript )
		RenameByOnHitscript( item, onHitScript, cursed );
	endif

	var dmg_mode := item.dmg_mod;
	if( dmg_mode )
		RenameByMod( item, "Damage", dmg_mode, cursed );
	endif

	var ar_mode := item.ar_mod;
	if( ar_mode )
		RenameByMod( item, "Ar", ar_mode, cursed );
	endif

	var ar_bonus := GetObjProperty( item, "ArBonus" );
	if( ar_bonus )
		RenameByMod( item, "ArBonus", CInt( ar_bonus * 5 ), cursed );
	endif

	var hp_mode := item.maxhp_mod;
	if( hp_mode )
		RenameByMod( item, "MaxHP", CInt( hp_mode / 2 ), cursed );
	endif

        var skill_mod := GetEquipmentModifiers( item );
        if( skill_mod.Size() )
                var skill_keys := skill_mod.Keys();
                foreach modifier in skill_keys
                        RenameByAttriMod( item, modifier, skill_mod[modifier], cursed );
                        SleepMS(5);
                endforeach
        endif

	foreach protection in immunities
                var mobile_prot := GetObjProperty( item, protection );
		if( mobile_prot )
			RenameByImmunity( item, protection, mobile_prot, cursed );
		endif
                SleepMS(5);
	endforeach
 
	foreach prot in wardings
                var charges := GetObjProperty( item, prot );
		if( charges )
			RenameByWardings( item, prot, cursed );
		endif
                SleepMS(5);
	endforeach

	foreach mod in regenerations
                var mobile_mod := GetObjProperty( item, mod );
		if( mobile_mod )
			RenameByImmunity( item, mod, mobile_mod, cursed );
		endif
                SleepMS(5);
	endforeach

	return 1;
endfunction

function RenameByHitscript( item, hitScript, cursed )

        var enchantment_cfg := ReadConfigFile( ":loot:effects_hit" );
        if( enchantment_cfg.errortext )
		PrintTextAbove( item, "Error reading config <:loot:effects_hit> -->"+enchantment_cfg.errortext );
		return 0;
        endif

	var chant_elem := FindConfigElem( enchantment_cfg, hitScript );
        if( chant_elem.errortext )
		PrintTextAbove( item, "Error reading chant_elem <"+hitScript+"> -->"+chant_elem.errortext );
		return 0;
        endif

	SetAdjective( item, chant_elem, "Name", cursed );
	SetObjProperty( item, "HitScript", chant_elem.HitScript );

        return 1;
endfunction

function RenameByOnHitscript( item, onHitScript, cursed )

        var enchantment_cfg := ReadConfigFile( ":loot:effects_onhit" );
        if( enchantment_cfg.errortext )
		PrintTextAbove( item, "Error reading config <:loot:effects_onhit> -->"+enchantment_cfg.errortext );
		return 0;
        endif

	var chant_elem := FindConfigElem( enchantment_cfg, onHitScript );
        if( chant_elem.errortext )
		PrintTextAbove( item, "Error reading chant_elem <"+onHitScript+"> -->"+chant_elem.errortext );
		return 0;
        endif

	SetAdjective( item, chant_elem, "Name", cursed );
	SetObjProperty( item, "OnHitScript", chant_elem.OnHitScript );

        return 1;
endfunction

function GetEnchantmentElem( item, elem )

        var enchantment_cfg := ReadConfigFile( ":loot:enchant_desc" );
        if( enchantment_cfg.errortext )
		PrintTextAbove( item, "Error reading config <:loot:enchant_desc> -->"+enchantment_cfg.errortext );
		return 0;
        endif

	var chant_elem := FindConfigElem( enchantment_cfg, elem );
        if( chant_elem.errortext )
		PrintTextAbove( item, "Error reading chant_elem <"+elem+"> -->"+chant_elem.errortext );
		return 0;
        endif

        return chant_elem;
endfunction

function RenameByAttriMod( item, attribute, value, cursed )

        if( !value )
                return 0;
        endif

        if( attribute in AP_ListAttributesByType( "Stat" ))
                value := CInt( value / 5 );
        endif

        var mod_elem := GetEnchantmentElem( item, attribute );
	var name := CStr ( "Name"+value );

	SetAdjective( item, mod_elem, name, cursed );

        return 1;
endfunction

function RenameByMod( item, mod, value, cursed )

        if( !value )
                return 0;
        endif

        var mod_elem := GetEnchantmentElem( item, mod );
        
	var temp := CInt( value / 5 );
        if( temp )
                var name := CStr( "Name"+temp );
                SetAdjective( item, mod_elem, name, cursed );
        endif

        return 1;
endfunction

function RenameByImmunity( item, immunity, power, cursed )

        var immunity_elem := GetEnchantmentElem( item, immunity );
	var name := CStr( "Name"+power );

	SetAdjective( item, immunity_elem, name, cursed );

        return 1;
endfunction

function RenameByWardings( item, immunity, cursed )

        var immunity_elem := GetEnchantmentElem( item, immunity );

	SetAdjective( item, immunity_elem, "Name", cursed );

        return 1;
endfunction

function SetAdjective( item, elem, value, cursed )

	var colorkey := "Color";

	if( cursed )
		value := "C"+value;
		colorkey := "C"+colorkey;
	endif

	var position := GetConfigString( elem, "Place" );
        if( position.errortext )
		PrintTextAbove( item, "Error reading position <Place> -->"+position.errortext );
		return 0;
        endif

	var adjective := GetConfigString( elem, value );
        if( adjective.errortext )
		PrintTextAbove( item, "Error reading adjective <"+value+"> -->"+adjective.errortext );
		return 0;
        endif

	if( position == "Prefix" )
		SetName( item, Lower( adjective+" "+item.desc ));
	else
		SetName( item, Lower( item.desc+" "+adjective ));
	endif
 
        var color_elem := GetConfigString( elem, colorkey );
	var color_item := CInt( color_elem );
	if( color_item )
		item.color := color_item;
	endif

        return 1;
endfunction
