/*
        Created by *Edwards
        For FantasiaShard.com

        Edwards@FantasiaShard.com

        2008-10-24
        
        Last Modifications 2008-12-12
             * Revised code and fixed few CInt issues
             * Fixed value "value" in SPELL_GetResist()
             
        Last Modifications 2008-12-29
             * SPELL_GetResist() replace evalint for magery if mobile is NPC
             
        Last Modifications 2009-01-18
             * Less chance to resist spell now
             * [FIXED] Must success on checkskill to resist
             
        Last Modifications 2009-02-07
             * Added toggle for resisting spells with SPELL_ApplyDamage()

        Last Modifications 2009-10-26
             * Changed how to handle protections with new functions in :equipment:protections

        Last Modifications 2009-10-26
             * Removed toggle for resisting spells with SPELL_ApplyDamage()
        
        Thanks to ZuluHotel and PolDistroTeam for this excellent ideas
*/
use uo;
use os;
use util;
use basic;

include ":attributes:attributes";
include ":classes:classes";
include ":damage:damage";
include ":combat:critical";
include ":equipment:protections";

const NPC_MODIFIER := 4;

function SPELL_GetEfficiency( mobile, value )

	if( GetObjProperty( mobile, MAGE ))
		value *= ClasseBonus( mobile, MAGE );
	elseif( GetObjProperty( mobile, WARRIOR ))
		value /= ClasseBonus( mobile, WARRIOR );
	endif

	var penalty := CInt( SPELL_GetPenalty( mobile ));
	if( penalty >= 100 )
		return 0;
	endif

	return CInt( value * ( 100 - penalty ) / 100 );
endfunction

function SPELL_GetPenalty( mobile )

	var penalty := 0;
	if( mobile.IsA( POLCLASS_NPC ))
                return 0;
        endif

        var cfg_desc := ReadConfigFile( ":*:itemdesc" );
        
	foreach item in ListEquippedItems( mobile )
                SleepMS(5);
                
                var cfg_penalty := cfg_desc[item.objtype].MagicPenalty;
		if( cfg_penalty )
			penalty += cfg_penalty;
			continue;
		endif
                
                var prop_penalty := GetObjProperty( item, "MagicPenalty" );
		if( prop_penalty )
			penalty += prop_penalty;
			continue;
		endif
	endforeach

	if( penalty > 100 )
		penalty := 100;
	endif

	return CInt( penalty );
endfunction

function SPELL_GetAmount( mobile )

	var value := CInt( AP_GetSkill( mobile, MAGERY ) / 10 ),
	    amount := SPELL_GetEfficiency( mobile, RandomInt(12) + value + 5 );

	if( !amount )
		amount := 1;
	endif

	return CInt( amount );
endfunction

function SPELL_GetDuration( mobile )

        var value := CInt( AP_GetSkill( mobile, MAGERY ) * 10 ),
	    duration := SPELL_GetEfficiency( mobile, value );
     
	if( !duration )
		duration := 1;
	endif

	return CInt( duration );
endfunction

function SPELL_MathDamage( mobile, circle, targRadius := 0 )

	if( targRadius )
                circle -= 3;
        endif
        
        if( circle < 1 )
                circle := 1;
        endif
        
        var modifier := CInt( AP_GetSkill( mobile, MAGERY ) / 5 ),
            amount := RandomDiceRoll(( circle * 3 )+"d5+"+modifier );
            
        if( amount > CInt( circle * ( 13 + circle )))
                amount := CInt( circle * ( 13 + circle ));
        endif

	amount := CInt( SPELL_GetEfficiency( mobile, amount ) / 5 );

	return amount;
endfunction

function SPELL_ApplyDamage( targ, amount, type, mobile, element:=DMG_MAGIC )

	amount := PROT_GetElementalDamage( targ, amount, element );
        amount := CriticalHit( mobile, targ, amount );
        
        ApplyDamageEX( targ, CInt( amount ), DMG_MAGIC + type, mobile );

        return amount;
endfunction

function SPELL_GetImmunity( mobile, targ, circle )

        if( GetObjProperty( targ, WARRIOR ))
                circle *= ClasseBonus( targ, WARRIOR );
        endif

        var perm_protection := GetMobileProtection( targ, "PermMagicProtection" );
        if( perm_protection >= circle )
		PrintTextAbovePrivate( targ, mobile.name+"'s spell is absorbed by your magical protection!", targ );
                if( mobile != targ )
		        PrintTextAbovePrivate( targ, "The spell dissipates upon contact with "+targ.name+"'s magical barrier!", mobile );
                endif
		return "Immuned";
        endif

        var targ_protection := GetMobileProtection( targ, "MagicProtection" ),
            item, charges, cursed;
            
	if( targ_protection )
		foreach thing in ListEquippedItems( targ )
			if( CInt( GetObjProperty( thing, "MagicProtection" )) == targ_protection )
				charges := CInt( GetObjProperty( item, "MagicProtection" ));
				if( charges <= 0 )
                                        SetMobileProtection( targ, "MagicProtection", 0 );
					EraseObjProperty( item, "MagicProtection" );
					PrintTextAbovePrivate( targ, item.desc+" is out of charges!", targ );
                                else
                                        item := thing;
                                        break;
				endif
			endif
                        SleepMS(5);
		endforeach
	endif

	if( charges >= circle )
		charges -= circle;
		SetObjProperty( item, "MagicProtection", charges );
  
                cursed := GetObjProperty( item, "Cursed" );
		if( !cursed )
			PrintTextAbovePrivate( targ, "Your "+item.desc+" protected you from the spell!", targ );
			return "Immuned";
		else
                        SendSysMessage( targ, "One of your equipped item increase the effect of the spell!" );
			return "Cursed";
		endif
	endif

	return 0;
endfunction

function SPELL_Reflected( mobile, targ, circle )

        if( GetObjProperty( targ, WARRIOR ))
                circle *= ClasseBonus( targ, WARRIOR );
        endif
        
	var targ_permreflection := GetMobileProtection( targ, "PermSpellReflection" );
	if( targ_permreflection >= circle )
                SendSysMessage( mobile, "The spell is reflected back to you!" );
		return "Reflected";
	endif
 
	var targ_reflection := GetMobileProtection( targ, "SpellReflection" ),
            item, charges, cursed;

	if( targ_reflection )
		foreach thing in ListEquippedItems( targ )
			if( CInt( GetObjProperty( thing, "SpellReflection" )) == targ_reflection )
				charges := CInt( GetObjProperty( item, "SpellReflection" ));
				if( charges <= 0 )
					SetMobileProtection( targ, "SpellReflection", 0 );
					EraseObjProperty( item, "SpellReflection" );
					PrintTextAbovePrivate( targ, item.desc+" is out of charges!", targ );
                                else
					item := thing;
                                        break;
				endif
			endif
                        SleepMS(5);
		endforeach
	endif

	if( charges >= circle )
		charges -= circle;
		SetObjProperty( item, "SpellReflection", charges );
		if( !cursed )
			PrintTextAbovePrivate( targ, "Your "+item.desc+" reflected the spell!", targ );
                        SendSysMessage( mobile, "The spell is reflected back to you!" );
			return "Reflected";
		else
			PrintTextAbovePrivate( targ, "Your "+item.desc+" prevent all reflections!", targ );
		endif
	endif
 
        return 0;
endfunction

function SPELL_GetResist( mobile, targ, circle, value )

	var mobile_magery := AP_GetSkill( mobile, MAGERY ),
	    mobile_evalint := AP_GetSkill( mobile, EVALUATING_INTELLIGENCE ),
	    targ_resist := AP_GetSkill( targ, RESISTING_SPELLS );
     
        if( mobile.IsA( POLCLASS_NPC ))
                 mobile_evalint := mobile_magery;
        endif
     
	var chance := CInt(( targ_resist - (( mobile_magery / 4 ) + ( circle * 3 ))) * 0.8 );

	if( GetObjProperty( targ, MAGE ))
		chance *= ClasseBonus( targ, MAGE );
	elseif( GetObjProperty( targ, WARRIOR ) )
                chance /= ClasseBonus( targ, WARRIOR );
	endif

	if( GetObjProperty( mobile, MAGE ))
		chance /= ClasseBonus( mobile, MAGE );
	endif
        
        if( SkillCheck( targ, RESISTING_SPELLS, CInt( circle * 8 )) > 0 )
                if( RandomInt(100) <= chance )
                        if( !GetObjProperty( targ, WARRIOR ))
                                SendSysMessage( targ, "You feel yourself resisting magical energy!" );
                                PlaySoundEffect( targ, 0x01e7 );
                                PlayObjectCenteredEffect( targ, 0x01e2, 5, 5 );
                                
		                value /= 2;
		                if( value < 1 )
			                value := 1;
		                endif
                        endif
                endif
        endif

	value *= CDbl(( mobile_evalint - targ_resist ) / 200 ) + 1;
        if( !targ.IsA( POLCLASS_NPC ))
                value *= 0.55;
        else
                value *= NPC_MODIFIER;
        endif
 
	if( GetObjProperty( targ, WARRIOR ) )
                value *= ClasseBonus( targ, WARRIOR );
	endif

	return CInt( value );
endfunction
