/* $Id: uo_extend.inc 535 2006-07-03 20:38:18Z austinheilman $
 *
 *
 */

use uo;
use os;
use util;
use polsys;
use cfgfile;
use math;

//
// For: EnumerateOnlineCharactersABC()
//
CONST SORT_ASCENDING	:= 0x0; // Default
CONST SORT_DESCENDING	:= 0x1;

// For: GetIsAType()
CONST ISA_GET_MAIN	:= 0x0; // Default
CONST ISA_GET_ALL	:= 0x1;

// For: ListMobilesInRange()
CONST RNG_SQUARE := 0x00; // Range is "square" (ie, simply +/- to x & y)
CONST RNG_CIRCLE := 0x01; // Range is a real "circle" [DEFAULT]


/*
 * EnumerateOnlineCharactersABC(dir)
 *
 * Purpose
 * Builds a list of online characters in ABC order.
 *
 * Parameters
 * dir:		Determines if the list should be made ascending or descending.
 * cmdlevel:	Command level filter. If the command level of a player is above
 *		the one passed, it will not be placed in the list.
 *		Use -1 to list everyone.
 *
 * Return Value:
 * Returns an array of mobile references.
 *
 */
function EnumerateOnlineCharactersABC(dir:=SORT_ASCENDING, cmdlevel:=-1)
	var dict := dictionary;
	foreach player in ( EnumerateOnlineCharacters() )
		if ( cmdlevel != -1 )
			if ( player.cmdlevel > cmdlevel )
				continue;
			endif
		endif

		var key := Upper(player.name);
		if ( !dict.Exists(key) )
			dict[key] := array{};
		endif
		dict[key].Append(player);
		SleepMS(2);
	endforeach

	var z_list := array{};
	foreach list in ( dict )
		foreach player in ( list )
			if ( dir == SORT_ASCENDING )
				z_list.Append(player);
			else
				z_list.Insert(1, player);
			endif
			SleepMS(2);
		endforeach
		SleepMS(2);
	endforeach

	return z_list;
endfunction

/*
 * EnumeratePhysicalItems(mobile)
 *
 * Purpose
 * Builds a list of equipped items on a mobile that are safe to access.
 *
 * Parameters
 * mobile:	Mobile to get the equipment on.
 *
 * Return Value:
 * Returns an array of item references.
 *
 */
function EnumeratePhysicalItems(mobile)
	var equip_list := array{};

	// Hair, beard, backpack, mount
	var skip_layers := array{11, 16, 21, 25};
	var i;

	for ( i:=0; i<=25; i+=1 )
		if ( i in skip_layers )
			continue;
		else
			var item := GetEquipmentByLayer(mobile, i);
			if ( item )
				equip_list.Append(item);
			endif
		endif
		SleepMS(2);
	endfor

	return equip_list;
endfunction

/*
 * GetIsAType(object, flags)
 *
 * Purpose
 * Retrieves the 'IsA' type for an object or all of the
 * classes it belongs to.
 *
 * Parameters:
 * object:	Anything belonging to the UObject class heirarchy.
 * flags:	Flags...
 *
 * Return value:
 * Returns an error on failure.
 * Returns an integer if ISA_GET_MAIN is passed for flags.
 * Returns an array of integers if ISA_GET_ALL is used.
 *
 */
function GetIsAType(object, flags:=ISA_GET_MAIN)
	var type_list := array;
	var i := 16; // Max number of IsA Id #s
	for ( i; i > 0; i-=1 )
		if ( !object.IsA(i) )
			continue;
		elseif ( flags & ISA_GET_ALL )
			type_list.Append(i);
		else
			return i;
		endif
		SleepMS(2);
	endfor

	if ( type_list.Size() > 0 )
		return type_list;
	else
		return error{"errortext":="Unable to determine IsA type of '"+TypeOf(object)+"'"};
	endif
endfunction

/*
 * GetIsATypeName(isa_id)
 *
 * Purpose
 * Retrieves the name for an IsA id number.
 *
 * Parameters:
 * isa_id:	Any number that matches an IsA number.
 *
 * Return value:
 * Returns an error on failure.
 * Returns a string on success.
 *
 */
function GetIsAName(isa_id)
	isa_id := CInt(isa_id);
	var type := "POLCLASS_";
	case ( isa_id )
		1: return type+"UOBJECT"; break;
		2: return type+"ITEM"; break;
		3: return type+"MOBILE"; break;
		4: return type+"NPC"; break;
		5: return type+"LOCKABLE"; break;
		6: return type+"CONTAINER"; break;
		7: return type+"CORPSE"; break;
		8: return type+"DOOR"; break;
		9: return type+"SPELLBOOK"; break;
		10: return type+"MAP"; break;
		11: return type+"MULTI"; break;
		12: return type+"BOAT"; break;
		13: return type+"HOUSE"; break;
		14: return type+"EQUIPMENT"; break;
		15: return type+"ARMOR"; break;
		16: return type+"WEAPON"; break;
		default: return error{"errortext":="'"+isa_id+"' is not a valid IsA number."};
	endcase
endfunction

function ListMobilesInRange( x, y, z, range, realm := _DEFAULT_REALM, flags := LISTEX_FLAG_NORMAL, range_type := RNG_SQUARE )

	if( range < 0 )
		return 0;
	endif

	// If passed normal flags and range is a square, then use the internal function
	if( !( flags & LISTEX_FLAG_CONCEALED ) && range_type == RNG_SQUARE )
		return ListMobilesNearLocationEx( x, y, z, range, flags, realm );
	endif

	if( !( flags & LISTEX_FLAG_NORMAL ))
		flags := flags | LISTEX_FLAG_NORMAL;
	endif

	// Validate the min/max values
	var min_range := { x-range, y-range };
	var max_range := { x+range, y+range };
 
	if( min_range[1] < 0 )
                min_range[1] := 0;
        endif

	if( min_range[2] < 0 )
                min_range[2] := 0;
        endif
        
        var map_maxX := 6143,
            map_maxY := 4095;
            
        if( realm == "britannia" )
                if( max_range[1] > map_maxX )
                       max_range[1] := map_maxX;
                endif
        
	        if( max_range[2] > map_maxY )
                       max_range[2] := map_maxY;
                endif
        endif

	set_script_option( SCRIPTOPT_NO_RUNAWAY, 1 );

	var mobiles := ListObjectsInBox( min_range[1], min_range[2], z-15, max_range[1], max_range[2], z+15, realm );
	var dTrig := dictionary;
	if( range_type == RNG_CIRCLE )
		// Set up the trig dictionary: key (x) to value (+y)
		var iHyp := Pow( range, 2 );
		for xpos := ( x-range ) to ( x+range )
			dTrig[xpos] := CInt( Ceil( Sqrt( iHyp - Pow( Abs( x-xpos ), 2 )))); //DeltaY
		endfor
	endif

	var fNrm := flags & LISTEX_FLAG_NORMAL;
	var fHid := flags & LISTEX_FLAG_HIDDEN;
	var fGho := flags & LISTEX_FLAG_GHOST;
	var fCon := flags & LISTEX_FLAG_CONCEALED;

	var valid_mob := array;
	foreach mob in mobiles
		if( mob.isa( POLCLASS_MOBILE ))
			if(( mob.hidden && fHid ) || ( mob.dead && fGho ) || ( mob.concealed == 0 && fCon ) ||
			   ( !mob.hidden && !mob.dead && !mob.concealed && fNrm ))
      
				if( range_type == RNG_CIRCLE )
					if(( mob.y >= ( y - dTrig[mob.x] )) && ( mob.y <= ( y + dTrig[mob.x] )))
						valid_mob.append( mob );
					endif
				else
					valid_mob.append( mob );
				endif
			endif
		endif
                SleepMS(5);
	endforeach

	set_script_option( SCRIPTOPT_NO_RUNAWAY, 0 );

	return valid_mob;
endfunction

function EquipFromTemplate2( mobile, elem_name:=0 );

	var index_cfg := ReadConfigFile( "::equip" );
 
	if( index_cfg.errortext )
		return error{ "errortext":="Could not open equipIndex.cfg -> "+index_cfg.errortext };
	elseif( !elem_name )
		var npc_elem := NPC_GetNPCConfig( mobile );
		elem_name := npc_elem.EquipTemplate;
	endif

	var index_elem := index_cfg[elem_Name];
	if( !index_elem )
		return error{ "errortext":="equipIndex.cfg elem ["+index_elem+"] does not exist." };
	endif

	foreach entry in ( GetConfigStringArray( index_elem, "Equip" ))
 
                var elem_string := SplitWords( entry );
                
		var item := CreateItemAtLocation( mobile.x, mobile.y, mobile.z, elem_string[1], 1, mobile.realm );
		item.movable := 1;
                item.color := CInt( elem_string[2] );

		if( item.errortext )
			PrintTextAbove( mobile, "Unable to create ["+item+"] ->"+item.errortext );
		elseif( !EquipItem( mobile, item ))
		        PrintTextAbove( mobile, "Unable to equip - "+item.desc+" - destroying." );
		        DestroyItem( item );
		endif
  
                SetObjProperty( item, "NoLoot", 1 );
                
		SleepMS(5);
	endforeach

	return 1;
endfunction

function FindMatchingObjtypeInContainer( container, item )

	var result;
        foreach object in EnumerateItemsInContainer( container );
                SleepMS(5);

                if( object.objtype != item.objtype )
                        continue;
                elseif( object.color != item.color )
                        continue;
                elseif( object.graphic != item.graphic )
                        continue;
                endif
                
                result := object;
                break;
        endforeach

	return result;
endfunction
